<!DOCTYPE html>
<html xmlns="http://www.w3.org/1999/xhtml" xml:lang="en" lang="en">
<head>

<title>Ontospy > {{docs_title}}</title>

<meta http-equiv="Content-Type" content="text/html;charset=utf-8" />



<style type="text/css" media="screen">


    /*STANDARD CSS */


    body {
        margin-left: 6em;
        margin-right: 12em;
        font-size: 14px;
        font-family: "Helvetica Neue", Helvetica;
    }

    h1, h2, h3, h4, h5, h6 { text-align: left }

    h1, h2, h3, h4, h5 { color: #000000;}
    h2 { font: 140% sans-serif }
    h3 { font: 120% sans-serif }
    h4 { font: bold 100% sans-serif }
    h5 { font: italic 100% sans-serif }
    h6 { font: small-caps 100% sans-serif }

    .margintop {
        margin-top: 40px;
    }


    a:link, a:visited {
        color: #666;
        text-decoration: none;
        }
    
    a:hover {
    color: #000
    }

    
    /*SLIDER CSS */


    .hidden {
        display: none;
    }

    
    .side-panel{
        background-color: #E6E6E6;
        padding: 10px;
        /*word-wrap: break-word;*/
        overflow: auto;
    }

    .side-panel-left{
        box-shadow: -5px 0 15px -6px #585858 inset;
    }

    .side-panel-right{
        box-shadow: 5px 0 15px -6px #585858 inset;
    }

    a.class {
        color: darkred;
    }
    a.prop {
        color: darkblue;
    }

    .turtlesource {
        font-size: 12px;
        overflow: auto;
    }

    dt {
        color: darkslategrey;
    }

    .text-muted {
        color: grey;
    }

    hr {
        margin-top: 20px;
        margin-bottom: 20px;
    }

    .credits {
        color: gray;
    }

    a.greylink {
        color: #666;
        text-decoration: underline;
    }

    .section-title {
        font-size: smaller;
    }

    .header {
        font-size: 15px;
        margin-bottom: 30px;
    }
    a.active {
        font-weight: bold;
        color: black;
    }


</style>



{% block custom_css %}


    
{% endblock custom_css %}






</head>


<body>

	{# VISIBLE PAGE HTML #}

	{# note: showpanel arg is the ID of the div that is showing.. #}

	<h1 class="title"><a href="javascript:showpanel('ontoinfo');">{{docs_title}}</a></h1>

	<hr>




    {% block main_content %}


    {% endblock main_content %}




	<hr>
    <small class="credits">Ontology documentation automatically generated with <a class="greylink" href="http://lambdamusic.github.io/Ontospy/" target="_blank">Ontospy</a> on {% now 'utc', '%a, %d %b %Y %H:%M:%S' %}.
    
        {% block extra_footer %}


        {% endblock extra_footer %}
    </small>



	{# SLIDER AND HIDDEN TABS #}

	{% include "shared/d3_hiddendivs.html" %}


	<div id='slider' class="side-panel side-panel-left">
		<span class="text-muted">Press <kbd>ESC</kbd> to close</span>
		<div id="slide_contents"></div>
	</div>

	{# <button id='trigger'>Trigger</button> #}


    



    {% block the_javascript %}


    {% endblock the_javascript %}





</body>
</html>
